/**********************************************************************/
/*                                                                    */
/*  Program: lorenz.c                                                 */
/*                                                                    */
/* Lorenz attractor                                                   */
/* ================                                                   */                  
/*                                                                    */
/* This program calculates an image of Lorenz attractor               */
/*                                                                    */
/* Parameters:                                                        */
/*                                                                    */
/* image: Output image name                                           */
/* size : Image size in pixels (512).                                 */
/* col  : Greyscale colour (0 <= col < 32768).                        */
/* n    : Number of iterations (10000).                               */
/*                                                                    */
/*                                                                    */
/* Program by:    Stefan Spaennare, Lund, Sweden                      */
/* E-mail:        stefan@spaennare.se                                 */
/*                                                                    */
/* First version: June 1995                                           */
/* Latest update: 1998-08-22                                          */
/*                                                                    */
/**********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/

#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <time.h>
#include "image.h"

double timetic=(double)(CLOCKS_PER_SEC);

double pi=3.14159265358979323;


void line(im,x1,y1,x2,y2,col)
short **im;
int x1,y1,x2,y2;
int col;
{
   double dx,dy,r,rr,ax,ay,x,y;
   int i,xi,yi,ri;

   dx=(double)(x2)-(double)(x1);
   dy=(double)(y2)-(double)(y1);

   r=sqrt(dx*dx+dy*dy);

   ri=2*(int)(r+1.0);

   rr=(double)(ri);

   ax=dx/rr;
   ay=dy/rr;

   x=(double)(x1);
   y=(double)(y1);

   for (i=0; i<=ri; i++) {
      xi=(int)(x+0.5);
      yi=(int)(y+0.5);
      im[yi][xi]=col;
      x=x+ax;
      y=y+ay;
   } /* for i */

} /* line */



int main(argc,argv)
int argc;
char *argv[];
{
   struct image_head header;    
   short **image;

   int  x72,y72,x7,y7,i,j,n,col,size;

   double v1,v2,c,r,dt,x,y,z,x1,y1,x0,y0,ss;
   double flops,ftime,sf;


   clock_t time1,timediff;


   if (argc<4) {
      printf("Usage: %s image sizei coli ni\n",argv[0]);
      exit(0);
   } /* if */

   size=atoi(argv[2]);
   col=atoi(argv[3]);
   n=atoi(argv[4]);

   header.id=image_magic;
   header.typ=im_s16;
   header.off=512;
   header.hgh=size;
   header.wid=size;
 
   image = (short **)calloc(header.hgh,sizeof(short*));

   CreateIm_s16(image,header.hgh,header.wid);


for (i=0; i<size; i++) {
   for (j=0; j<size; j++) {
       image[i][j]=32767;
    } /* for j */
} /* for i */

time1=clock();


   v1=0.0; v2=90.0;
   v1=cos(v1*pi/180.0);
   v2=cos(v2*pi/180.0);

   sf=(double)(size)/1024.0;

   c=-10.0*sf;
   r=28.0;
   dt=0.005;
   x0=512.0*sf;
   y0=712.0*sf;

   ss=8.0/3.0;
   x=1.0; y=1.0; z=1.0;
   x72=(int)(x0+0.5); y72=(int)(y0+0.5);

for (i=1; i<=n; i++) {
     x1=x;
     y1=y;

     x=x+10.0*(y-x)*dt;
     y=y+(-x1*z+r*x1-y)*dt;
     z=z+(x1*y1-ss*z)*dt;

     x7=(int)(c*(x+y*v1)+x0+0.5);
     y7=(int)(c*(z+y*v2)+y0+0.5);

     line(image,x72,y72,x7,y7,col);

     x72=x7;
     y72=y7;

} /* for i */

timediff=clock()-time1;

ftime=(double)(timediff)/timetic;
printf("\nTime:  %12.2f\n",ftime);

WriteImage(argv[1],image,header,im_s16);

} /* End */
